/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ExportAction.java,v 1.1 2004/04/28 16:30:08 dwagelaa Exp $
 * $Log: ExportAction.java,v $
 * Revision 1.1  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 */

package be.ac.vub.cocompose.eclipse.actions;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.CoComposeEditor;
import be.ac.vub.cocompose.eclipse.draw2d.EPSGraphics;
import be.ac.vub.cocompose.log.Log;

/**
 * Exports the current model in another file format. Currently only supports
 * EPS. 
 * @author Dennis Wagelaar
 * @version 0.2.9, 28/04/2004
 * @since 0.2.9
 */
public class ExportAction extends WorkbenchPartAction
{
  private Log log = CoCompose.getDefault().getLog();
  private IFile file = null;
  
  /**
   * Creates a CoComposePasteAction.
   * @param part the CoCompose editor.
   */
  public ExportAction(IWorkbenchPart part)
  {
    super(part);
  }

  /**
   * @see org.eclipse.jface.action.IAction#run()
   */
  public void run()
  {
    CoComposeEditor editor = (CoComposeEditor) getWorkbenchPart();
    SaveAsDialog dialog = new SaveAsDialog(editor.getSite().getWorkbenchWindow().getShell());
    file = ((IFileEditorInput) editor.getEditorInput()).getFile();
    String filename = file.getName();
    if (filename.indexOf('.') > -1)
      filename = filename.substring(0, filename.indexOf('.'));
    dialog.setOriginalName(filename);
    dialog.open();
    IPath path = dialog.getResult();
    if (path == null)
      return;
    // set new URI
    IWorkspace workspace = ResourcesPlugin.getWorkspace();
    file = workspace.getRoot().getFile(path);
    if (!file.getName().endsWith(".eps"))
    {
      log.report(new IOException("Don't know how to export to " + 
        file.getName() + "; currently only EPS format is supported."));
      return;
    }
    // build Save As operation
    WorkspaceModifyOperation op = new WorkspaceModifyOperation() 
    {
      public void execute(final IProgressMonitor monitor) throws CoreException 
      {
        try 
        {
          performSaveEPS(monitor);
        } 
        catch (Exception e) 
        {
          log.report(e);
        }
      }
    };
    // execute Save As and reload file in editor
    try 
    {
      new ProgressMonitorDialog(
        editor.getSite().getWorkbenchWindow().getShell()).run(false, true, op);
      getCommandStack().markSaveLocation();
    } 
    catch (Exception e) 
    {
      log.report(e);
    } 
  }

  /**
   * Performs saving an EPS file.
   * @param monitor
   */
  public void performSaveEPS(IProgressMonitor monitor)
  {
    CoComposeEditor editor = (CoComposeEditor) getWorkbenchPart();
    // Generate EPS
    String name = getFile().getName();
    if (name.indexOf('.') > -1)
      name = name.substring(0, name.indexOf('.'));
    try
    {
      EPSGraphics g = new EPSGraphics(name);
      AbstractGraphicalEditPart part =
        (AbstractGraphicalEditPart) editor
          .getGraphicalViewer()
          .getRootEditPart();
      part.getFigure().paint(g);
      char[] eps = g.toString().toCharArray();
      byte[] beps = new byte[eps.length];
      for (int i = 0; i < eps.length; i++)
      {
        beps[i] = (byte) eps[i];
      }
      ByteArrayInputStream input = new ByteArrayInputStream(beps);
      if (file.exists())
        file.setContents(input, true, false, monitor);
      else
        file.create(input, true, monitor);
      g.dispose();
    }
    catch (Exception e)
    {
      log.report(e);
    }
  }

  /**
   * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
   */
  protected boolean calculateEnabled()
  {
    return (getWorkbenchPart() instanceof CoComposeEditor);
  }
  
  /**
   * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
   */
  protected void init()
  {
    setId(GEFActionConstants.EXPORT);
    setText("&Export...");
    setToolTipText("Export to external format...");
  }
  
  /**
   * Gets the file to which the EPS is exported.
   * @return the file to which the EPS is exported.
   */
  public IFile getFile()
  {
    return file;
  }
}
