/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.editparts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;

import be.ac.vub.cocompose.eclipse.editpolicies.CoComposeConnectionEditPolicy;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.Relationship;

/**
 * Eclipse editpart controller for EclipseRelationship objects.
 * @author Dennis Wagelaar
 * @version 0.2.0, 12/12/2003
 * @since 0.2.0
 */
public class RelationshipEditPart extends AbstractConnectionEditPart
  implements PropertyChangeListener
{
  /**
   * Activates the editpart to listen to changes in the model.
   * @see org.eclipse.gef.EditPart#activate()
   */
  public void activate()
  {
    super.activate();
    getRelationship().addPropertyChangeListener(this);
  }

  /**
   * Makes the EditPart insensible to changes in the model
   * by removing itself from the model's list of listeners.
   * @see org.eclipse.gef.EditPart#deactivate()
   */
  public void deactivate()
  {
    super.deactivate();
    getRelationship().removePropertyChangeListener(this);
  }

  /**
   * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#activateFigure()
   */
  public void activateFigure()
  {
    super.activateFigure();
    /* Once the figure has been added to the ConnectionLayer, start listening for its
     * router to change.
     */
    getFigure().addPropertyChangeListener(Connection.PROPERTY_CONNECTION_ROUTER, this);
  }

  /**
   * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#deactivateFigure()
   */
  public void deactivateFigure()
  {
    getFigure().removePropertyChangeListener(Connection.PROPERTY_CONNECTION_ROUTER, this);
    super.deactivateFigure();
  }

  /**
   * Listens to changes in properties of the Wire (like the
   * contents being carried), and reflects is in the visuals.
   * @param evt - event notifying the change.
   * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
   */
  public void propertyChange(PropertyChangeEvent evt) 
  {
    String id = evt.getPropertyName();
    if (Connection.PROPERTY_CONNECTION_ROUTER.equals(id))
    {
      refreshBendpoints();
//      refreshBendpointEditPolicy();
    }
    else if (Properties.ID_BENDPOINT.equals(id))
    {
      refreshBendpoints();       
    }
  }

  /**
   * Updates the bendpoints, based on the model.
   */
  protected void refreshBendpoints() 
  {
//    if (getConnectionFigure().getConnectionRouter() instanceof ManhattanConnectionRouter)
//      return;
//    List modelConstraint = getRelationship().getBendpoints();
//    List figureConstraint = new ArrayList();
//    for (int i=0; i<modelConstraint.size(); i++) 
//    {
//      WireBendpoint wbp = (WireBendpoint)modelConstraint.get(i);
//      RelativeBendpoint rbp = new RelativeBendpoint(getConnectionFigure());
//      rbp.setRelativeDimensions(wbp.getFirstRelativeDimension(),
//                    wbp.getSecondRelativeDimension());
//      rbp.setWeight((i+1) / ((float)modelConstraint.size()+1));
//      figureConstraint.add(rbp);
//    }
//    getConnectionFigure().setRoutingConstraint(figureConstraint);
  }

  /**
   * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
   */
  protected void createEditPolicies()
  {
    installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE, new ConnectionEndpointEditPolicy());
    //Note that the Connection is already added to the diagram and knows its Router.
//    refreshBendpointEditPolicy();
    installEditPolicy(EditPolicy.CONNECTION_ROLE, new CoComposeConnectionEditPolicy());
  }

  /**
   * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
   */
  protected IFigure createFigure() 
  {
    if (getRelationship() == null)
      return null;
    return new PolylineConnection();
  }

  /**
   * Returns the model associated with this.
   * @return the model associated with this.
   */
  protected Relationship getRelationship()
  {
    return (Relationship) getModel();
  }
}
