/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ModelElementPropertySource.java,v 1.4 2004/05/10 14:26:23 dwagelaa Exp $
 * $Log: ModelElementPropertySource.java,v $
 * Revision 1.4  2004/05/10 14:26:23  dwagelaa
 * worked on shared models and eclipse property sources
 *
 * Revision 1.3  2004/04/30 16:24:55  dwagelaa
 * + added ElementConstraint support for the CoCompose2 editor
 *
 */

package be.ac.vub.cocompose.eclipse.model.datatypes;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.internal.runtime.Assert;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.log.Log;

/**
 * Eclipse property source for a ModelElement.
 * @author Dennis Wagelaar
 * @version 0.2.10, 10/05/2004
 * @since 0.2.4
 */
public class ModelElementPropertySource extends ModelElementsPropertySource
{
  private IPropertyDescriptor[] desc = null;
  private ModelElement subject = null; 
  private ModelElement value = null;
  private Object id;
  private String name;
  
  /**
   * Creates a PartOfPropertySource
   * @param subject the subject for which to set the property.
   * @param value the current property value.
   * @param namespace the Namespace in which to look for the property.
   * @param elementClass the Class restriction for possible values.
   * @param id the property id.
   * @param name the property name to be displayed.
   */
  public ModelElementPropertySource(
    ModelElement subject,
    ModelElement value,
    Namespace namespace,
    Class elementClass,
    Object id,
    String name)
  {
    super(subject, new Vector(), namespace, elementClass);
    Assert.isNotNull(id);
    Assert.isNotNull(name);
    this.subject = subject;
    this.value = value;
    this.id = id;
    this.name = name;
  }

  /**
   * Creates a PartOfPropertySource
   * @param subject the subject for which to set the property.
   * @param value the current property value.
   * @param elementClass the Class restriction for possible values.
   * @param id the property id.
   * @param name the property name to be displayed.
   */
  public ModelElementPropertySource(
    ModelElement subject,
    ModelElement value,
    Class elementClass,
    Object id,
    String name)
  {
    super(subject, new Vector(), elementClass);
    Assert.isNotNull(id);
    Assert.isNotNull(name);
    this.subject = subject;
    this.value = value;
    this.id = id;
    this.name = name;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
   */
  public Object getPropertyValue(Object id)
  {
    ModelElement value = getValue();
    try
    {
      if (value != null)
        return new Integer(getPossibleValues().indexOf(value) + 1);
      else
        return new Integer(0);
    }
    catch (NumberFormatException e)
    {
      getLog().report(e);
    }
    return null;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
   */
  public void resetPropertyValue(Object id)
  {
    setValue(null);
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
   */
  public void setPropertyValue(Object id, Object value)
  {
    try
    {
      int valueIndex = ((Integer) value).intValue();
      if (valueIndex > 0)
        setValue((ModelElement) getPossibleValues().get(valueIndex - 1));
      else
        setValue(null);
    }
    catch (NumberFormatException e)
    {
      getLog().report(e);
    }
  }
 
  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    ModelElement value = getValue();
    if (value != null)
      return getFullName(value);
    else
      return "<<none>>";
  }
  
  /**
   * Gets the value for this property.
   * @return the value for this property.
   */
  public ModelElement getValue()
  {
    return value;
  }

  /**
   * Creates the property descriptors list
   * @return the property descriptors list
   */
  protected List createPropertyDescriptors()
  {
    List descs = new Vector();
    String[] values = createStringArray(getValueStrings());
    ComboBoxPropertyDescriptor desc = new ComboBoxPropertyDescriptor(id, name, values);
    desc.setLabelProvider(new IndexedLabelProvider(getPossibleValues()));
    descs.add(desc);
    return descs;
  }

  /**
   * Sets the value for this property.
   * @param value the value for this property.
   */
  protected void setValue(ModelElement value)
  {
    removeSharedModel();
    this.value = value;
    addSharedModel();
  }

  /**
   * Removes any shared model from the pool if the current value is in another
   * model.
   */
  private void removeSharedModel()
  {
    if (getValue() == null)
      return;
    Model model = subject.getModel();
    Model emodel = getValue().getModel();
    if ((model == null) || (emodel == null))
      return;
    if (model.equals(emodel))
      return;
    getSettings().removeSharedModel(emodel);
  }
  
  /**
   * Adds any shared model to the pool if the current value is in another
   * model.
   */
  private void addSharedModel()
  {
    if (getValue() == null)
      return;
    Model model = subject.getModel();
    Model emodel = getValue().getModel();
    if ((model == null) || (emodel == null))
      return;
    if (model.equals(emodel))
      return;
    getSettings().addSharedModel(emodel);
  }
}
