/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: XMIElementWriter.java,v 1.7 2004/05/10 14:26:23 dwagelaa Exp $
 * $Log: XMIElementWriter.java,v $
 * Revision 1.7  2004/05/10 14:26:23  dwagelaa
 * worked on shared models and eclipse property sources
 *
 * Revision 1.6  2004/05/07 16:10:21  dwagelaa
 * + added relative URI support
 *
 * Revision 1.5  2004/05/07 15:25:47  dwagelaa
 * fixed all things to compile again
 *
 */

package be.ac.vub.cocompose.io.xmi;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Iterator;
import org.xml.sax.SAXException;
import org.xml.sax.ContentHandler;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.ImplementationGenerator;
import be.ac.vub.cocompose.lang.core.ImplementationRolePart;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.Refinement;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.constraints.CompositeConstraint;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.properties.ModelProperty;
import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.log.Log;

/**
 * Writes a CoCompose model to an XMI file.
 * @author Dennis Wagelaar 
 * @version 0.2.10, 11/05/2004
 * @since 0.2.0
 */
public class XMIElementWriter extends DefaultSuperModelVisitor 
{
  private ContentHandler xmlHandler = null;
  private XMIElementAttributesWriter attributes =
    new XMIElementAttributesWriter();

  /**
   * Sets the XML handler.
   * @param xmlHandler - the XML handler.
   */
  public void setXmlHandler(ContentHandler xmlHandler)
  {
    this.xmlHandler = xmlHandler;
  }

  /**
   * Gets the XML handler.
   * @return the XML handler.
   */
  public ContentHandler getXmlHandler()
  {
    return xmlHandler;
  }

  /**
   * Visits a Namespace.
   * @param namespace - the element to visit.
   * @exception ModelElementException
   */
  public void visitNamespace(Namespace namespace) throws ModelElementException
  {
    try
    {
      current.visitModelElement(namespace);
      Iterator ownedElts = namespace.getOwnedElements().iterator();
      while (ownedElts.hasNext())
      {
        ModelElement elt = (ModelElement) ownedElts.next();
        attributes.visit(elt);
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "ownedElement", 
          XMI.CoCompose.qName("ownedElement"), attributes.getAttributes());
        elt.accept(current);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "ownedElement", 
          XMI.CoCompose.qName("ownedElement"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, namespace, Log.ERROR);
    }
  }

  /**
   * Visits a Model.
   * @param model - the element to visit.
   * @exception ModelElementException
   */
  public void visitModel(Model model) throws ModelElementException
  {
    try
    {
      attributes.visit(model);
      xmlHandler.startElement(XMI.CoCompose.NS_URI, "Model", 
        XMI.CoCompose.qName("Model"), attributes.getAttributes());
      super.visitModel(model);
      xmlHandler.endElement(XMI.CoCompose.NS_URI, "Model", 
        XMI.CoCompose.qName("Model"));
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, model, Log.ERROR);
    }
  }

  /**
   * Visits a Relationship.
   * @param relationship - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationship(Relationship relationship) throws ModelElementException
  {
    try
    {
      super.visitRelationship(relationship);
      // first participant
      insertReference("participant", relationship, relationship.getSourceParticipant());
      // second participant
      insertReference("participant", relationship, relationship.getTargetParticipant());
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, relationship, Log.ERROR);
    }
  }

  /**
   * Visits a RelationElement.
   * @param relationElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationElement(RelationElement relationElement) throws ModelElementException
  {
    try
    {
      super.visitRelationElement(relationElement);
      AttributesImpl atts = new AttributesImpl();
      Iterator relations = relationElement.getRelationships().iterator();
      while (relations.hasNext())
      {
        Relationship relation = (Relationship) relations.next();
        insertReference("relationship", relationElement, relation);
      }
      // xmi:Extension
      xmlHandler.startElement(XMI.NS_URI, "Extension", XMI.qName("Extension"), null);
      // Diagram:GraphicElement
      xmlHandler.startElement(XMI.Diagram.NS_URI, "GraphicElement", XMI.Diagram.qName("GraphicElement"), null);
      // position
      atts.clear();
      atts.addAttribute(XMI.Diagram.NS_URI, "x", XMI.Diagram.qName("x"), "CDATA", new Integer(relationElement.getPosition().x).toString());
      atts.addAttribute(XMI.Diagram.NS_URI, "y", XMI.Diagram.qName("y"), "CDATA", new Integer(relationElement.getPosition().y).toString());
      xmlHandler.startElement(XMI.Diagram.NS_URI, "position", XMI.Diagram.qName("position"), atts);
      xmlHandler.endElement(XMI.Diagram.NS_URI, "position", XMI.Diagram.qName("position"));
      // color
      atts.clear();
      atts.addAttribute(XMI.Diagram.NS_URI, "r", XMI.Diagram.qName("r"), "CDATA", new Integer(relationElement.getBackgroundColor().getRed()).toString());
      atts.addAttribute(XMI.Diagram.NS_URI, "g", XMI.Diagram.qName("g"), "CDATA", new Integer(relationElement.getBackgroundColor().getGreen()).toString());
      atts.addAttribute(XMI.Diagram.NS_URI, "b", XMI.Diagram.qName("b"), "CDATA", new Integer(relationElement.getBackgroundColor().getBlue()).toString());
      xmlHandler.startElement(XMI.Diagram.NS_URI, "color", XMI.Diagram.qName("color"), atts);
      xmlHandler.endElement(XMI.Diagram.NS_URI, "color", XMI.Diagram.qName("color"));
      // close elements
      xmlHandler.endElement(XMI.Diagram.NS_URI, "GraphicElement", XMI.Diagram.qName("GraphicElement"));
      xmlHandler.endElement(XMI.NS_URI, "Extension", XMI.qName("Extension"));
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, relationElement, Log.ERROR);
    }
  }

  /**
   * Visits a SolutionPattern.
   * @param solutionPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitSolutionPattern(SolutionPattern solutionPattern) throws ModelElementException
  {
    try
    {
      super.visitSolutionPattern(solutionPattern);
      if (solutionPattern.getSolutionModel() != null)
        insertReference("solutionModel", solutionPattern, solutionPattern.getSolutionModel());
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, solutionPattern, Log.ERROR);
    }
  }

  /**
   * Visits an ImplementationGenerator.
   * @param implementationGenerator - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationGenerator(ImplementationGenerator implementationGenerator) throws ModelElementException
  {
    try
    {
      super.visitImplementationGenerator(implementationGenerator);
      Iterator roleparts = implementationGenerator.getRoleParts().iterator();
      while (roleparts.hasNext())
      {
        ImplementationRolePart rp = (ImplementationRolePart) roleparts.next();
        insertReference("rolePart", implementationGenerator, rp);
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, implementationGenerator, Log.ERROR);
    }
  }

  /**
   * Visits an ImplementationPattern.
   * @param implementationPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationPattern(ImplementationPattern implementationPattern) throws ModelElementException
  {
    try
    {
      super.visitImplementationPattern(implementationPattern);
      if (implementationPattern.getContent() != null)
      {
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "content", XMI.CoCompose.qName("content"), null);
        xmlHandler.characters(implementationPattern.getContent().toCharArray(), 0,
          implementationPattern.getContent().length());
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "content", XMI.CoCompose.qName("content"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, implementationPattern, Log.ERROR);
    }
  }

  /**
   * Visits a RefinedElement.
   * @param refinedElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
  {
    try
    {
      super.visitRefinedElement(refinedElement);
      Model model = getModel(refinedElement);
      Iterator refinements = refinedElement.getRefinements().iterator();
      while (refinements.hasNext())
      {
        Refinement ref = (Refinement) refinements.next();
        insertReference("refinement", refinedElement, ref);
      }
      Iterator inhElts = refinedElement.getInheritsFrom().iterator();
      while (inhElts.hasNext())
      {
        RefinedElement elt = (RefinedElement) inhElts.next();
        insertReference("inheritsFrom", refinedElement, elt);
      }
      Iterator superElts = refinedElement.getSuperimposesOn().iterator();
      while (superElts.hasNext())
      {
        RefinedElement elt = (RefinedElement) superElts.next();
        insertReference("superimposesOn", refinedElement, elt);
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, refinedElement, Log.ERROR);
    }
  }

  /**
   * Visits a concept.
   * @param concept - the element to visit.
   * @exception ModelElementException
   */
  public void visitConcept(Concept concept) throws ModelElementException
  {
    try
    {
      super.visitConcept(concept);
      Iterator partOf = concept.getPartOf().iterator();
      while (partOf.hasNext())
      {
        Role role = (Role) partOf.next();
        insertReference("partOf", concept, role);
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, concept, Log.ERROR);
    }
  }

  /**
   * Visits a Role.
   * @param role - the element to visit.
   * @exception ModelElementException
   */
  public void visitRole(Role role) throws ModelElementException
  {
    try
    {
      super.visitRole(role);
      attributes.visit(role.getMultiplicity());
      xmlHandler.startElement(XMI.CoCompose.NS_URI, "multiplicity", XMI.CoCompose.qName("multiplicity"), attributes.getAttributes());
      role.getMultiplicity().accept(current);
      xmlHandler.endElement(XMI.CoCompose.NS_URI, "multiplicity", XMI.CoCompose.qName("multiplicity"));
      Iterator parts = role.getParts().iterator();
      while (parts.hasNext())
      {
        Concept concept = (Concept) parts.next();
        insertReference("part", role, concept);
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, role, Log.ERROR);
    }
  }

  /**
   * Visits an ElementConstraint.
   * @param elementConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitElementConstraint(ElementConstraint elementConstraint) throws ModelElementException
  {
    try
    {
      super.visitElementConstraint(elementConstraint);
      Iterator constrElts = elementConstraint.getConstrainedElements().iterator();
      while (constrElts.hasNext())
      {
        ModelElement constrElt = (ModelElement) constrElts.next();
        insertReference("constrainedElement", elementConstraint, constrElt);
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, elementConstraint, Log.ERROR);
    }
  }

  /**
   * Visits a CompositeConstraint.
   * @param compositeConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitCompositeConstraint(CompositeConstraint compositeConstraint) throws ModelElementException
  {
    try
    {
      super.visitCompositeConstraint(compositeConstraint);
      Iterator constr = compositeConstraint.getOwnedConstraints().iterator();
      while (constr.hasNext())
      {
        Constraint c = (Constraint) constr.next();
        insertReference("ownedConstraint", compositeConstraint, c);
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, compositeConstraint, Log.ERROR);
    }
  }

  /**
   * Visits a ModelProperty.
   * @param modelProperty - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelProperty(ModelProperty modelProperty) throws ModelElementException
  {
    try
    {
      super.visitModelProperty(modelProperty);
      if (modelProperty.getTargetModel() != null)
      {
        insertReference("targetModel", modelProperty, modelProperty.getTargetModel());
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, modelProperty, Log.ERROR);
    }
  }

  /**
   * Gets the root attributes to add to the first element.
   * @return the root attributes.
   */
  public Attributes getRootAttributes()
  {
    return attributes.getRootAttributes();
  }

  /**
   * Sets the root attributes to add to the first element.
   * @param rootAttributes - the root attributes.
   */
  public void setRootAttributes(Attributes rootAttributes)
  {
    attributes.setRootAttributes(rootAttributes);
  }

  /**
   * Sets a stacked visitor for this and the contained visitors.
   * @param stacked - the stacked visitor.
   */
  public void setStacked(ModelVisitor stacked)
  {
    super.setStacked(stacked);
    attributes.setStacked(stacked);
  }
  
  /**
   * Inserts an XMI tag that refers to the given model element. This can be
   * an idref tag or an href tag.
   * @param refName the name of the reference.
   * @param from the origin of the reference.
   * @param to the target of the reference.
   * @throws ModelElementException if the reference could not be made.
   * @throws SAXException if xml could not be created.
   */
  protected void insertReference(
    String refName,
    ModelElement from,
    ModelElement to)
    throws ModelElementException, SAXException
  {
    Model model = from.getModel();
    Model emodel = to.getModel();
    if ((model == null) || (emodel == null))
      throw new ModelElementException(
        "Source and target element must have a model", 
        from);
    if (model.equals(emodel))
      insertIDRef(refName, to);
    else
      insertHref(refName, model, to);
  }
  
  /**
   * Inserts an XMI idref tag to the given model element.
   * @param refName the reference tag name.
   * @param to the target model element.
   * @throws SAXException
   */
  private void insertIDRef(String refName, ModelElement to) throws SAXException
  {
    AttributesImpl atts = new AttributesImpl();
    atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "CDATA", to.getId());
    xmlHandler.startElement(XMI.CoCompose.NS_URI, refName, XMI.CoCompose.qName(refName), atts);
    xmlHandler.endElement(XMI.CoCompose.NS_URI, refName, XMI.CoCompose.qName(refName));
  }
  
  /**
   * Inserts an XMI href tag to the given model element.
   * @param refName the reference tag name.
   * @param from the model from which the reference is made.
   * @param to the target model element.
   * @throws ModelElementException if the reference could not be made.
   * @throws SAXException
   */
  private void insertHref(String refName, Model from, ModelElement to)
    throws ModelElementException, SAXException
  {
    Model emodel = to.getModel();
    if (emodel == null)
      throw new ModelElementException("Target element must have a model", to);
    String uri = getRelativeUri(emodel.getUri(), from.getUri());
    AttributesImpl atts = new AttributesImpl();
    atts.addAttribute(XMI.NS_URI, "href", XMI.qName("href"), "CDATA", uri + '#' + to.getId());
    xmlHandler.startElement(XMI.CoCompose.NS_URI, refName, XMI.CoCompose.qName(refName), atts);
    xmlHandler.endElement(XMI.CoCompose.NS_URI, refName, XMI.CoCompose.qName(refName));
  }
  
  /**
   * Gets the model containing the given element. 
   * @param element the element for which to get the model.
   * @return the model containing the given element.
   * @throws ModelElementException if the element has no model.
   */
  private Model getModel(ModelElement element) throws ModelElementException
  {
    Model model = element.getModel();
    if (model == null)
    {
      throw new ModelElementException(
        "Model must be set for each element when writing to XMI",
        element);
    }
    return model;
  }
  
  /**
   * Gets the uri relative to the base uri.
   * @param uri the absolute uri.
   * @param baseuri the base uri.
   * @return the relative uri.
   * @throws ModelElementException if no valid URIs are given.
   */
  private String getRelativeUri(String uri, String baseuri)
    throws ModelElementException
  {
    uri = uri.replaceAll(" ", "%20");
    baseuri = baseuri.replaceAll(" ", "%20");
    try
    {
      int indexeq = 0;
      while ((indexeq < uri.length()) && (indexeq < baseuri.length()) &&
             (uri.charAt(indexeq) == baseuri.charAt(indexeq)))
      {
        indexeq++;
      }
      String common = baseuri.substring(0, indexeq);
      common = common.substring(0, common.lastIndexOf('/') + 1);
      StringBuffer dots = new StringBuffer();
      while (indexeq < baseuri.length())
      {
        if (baseuri.charAt(indexeq) == '/')
        {
          dots.append("../");
        }
        indexeq++;
      }
      URI relUri = new URI(uri);
      URI baseUri = new URI(common);
      relUri = baseUri.relativize(relUri);
      dots.append(relUri.toString());
      if (relUri.isAbsolute())
        return relUri.toString();
      else
        return dots.toString();
    }
    catch (URISyntaxException e)
    {
      throw new ModelElementException(e);
    }
  }
}
