/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: XMIModelLinkReader.java,v 1.5 2004/05/10 14:26:23 dwagelaa Exp $
 * $Log: XMIModelLinkReader.java,v $
 * Revision 1.5  2004/05/10 14:26:23  dwagelaa
 * worked on shared models and eclipse property sources
 *
 * Revision 1.4  2004/05/07 16:10:21  dwagelaa
 * + added relative URI support
 *
 * Revision 1.3  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 */

package be.ac.vub.cocompose.io.xmi;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.io.ModelReader;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.properties.ModelProperty;

/** 
 * Reads the links in a CoCompose model from an XMI file (requires first pass by
 * XMIModelReader). 
 * @author Dennis Wagelaar 
 * @version 0.2.10, 11/05/2004
 * @since 0.2.0
 */
public class XMIModelLinkReader extends DefaultHandler 
{
  private CoCompose settings = CoCompose.getDefault();
  private Model model = null;
  private Map elementMap = null;
  private Namespace namespace = null;
  private ModelElement element = null;
  private Relationship relationship = null;
  private RefinedElement refinedElement = null;
  private Role role = null;
  private Concept concept = null;
  private SolutionPattern solutionPattern = null;
  private ModelProperty modelProperty = null;
  private XMIElementLinkAttributesReader elementReader = null;

  /**
   * Creates an XMIModelLinkReader
   * @param model the model after one pass by XMIModelReader
   * @param elementMap the map of id -&gt; element
   */
  public XMIModelLinkReader(Model model, Map elementMap)
    throws ModelElementException
  {
    if (model == null)
      throw new ModelElementException("Model may not be null");
    if (elementMap == null)
      throw new ModelElementException("ElementMap may not be null");
    setModel(model);
    setNamespace(model);
    setElementMap(elementMap);
    setElementReader(new XMIElementLinkAttributesReader(elementMap));
  }

  //
  // ContentHandler methods
  //

  /**
   * Processing instruction.
   *
   * @param target Processing instruction element name.
   * @param data Processing instruction element data.
   */
  public void processingInstruction(String target, String data)
    throws SAXException
  {
  } // processingInstruction

  /**
   * Start document.
   */
  public void startDocument() throws SAXException
  {
  } //startDocument

  /**
   * Start element.
   * @param uri XML namespace URI.
   * @param local local element name.
   * @param raw raw (qualified) element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startElement(String uri, String local, String raw,
    Attributes attrs) throws SAXException
  {
    if (uri == null)
      throw new SAXException("XML namespace URI may not be null");
    if (uri.equals(XMI.NS_URI))
      startXMIElement(local, attrs);
    else
      startCoComposeElement(local, attrs);
  } //startElement

  /**
   * Start XMI element.
   * @param local local element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startXMIElement(String local, Attributes attrs) throws SAXException
  {
  }

  /**
   * Start CoCompose element.
   * @param local local element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startCoComposeElement(String local, Attributes attrs) throws SAXException
  {
    try
    {
      if (local.equals("Model"))
      {
        getElementReader().setAttributes(attrs);
        getElementReader().visit(getModel());
      }
      else if (local.equals("ownedElement"))
      {
        startOwnedElement(attrs);
      }
      else if (local.equals("participant"))
      {
        startParticipant(attrs);
      }
      else if (local.equals("partOf"))
      {
        startPartOf(attrs);
      }
      else if (local.equals("part"))
      {
        startPart(attrs);
      }
      else if (local.equals("inheritsFrom"))
      {
        startInheritsFrom(attrs);
      }
      else if (local.equals("superimposesOn"))
      {
        startSuperimposesOn(attrs);
      }
      else if (local.equals("solutionModel"))
      {
        startSolutionModel(attrs);
      }
      else if (local.equals("targetModel"))
      {
        startTargetModel(attrs);
      }
    }
    catch (ModelElementException e)
    {
      SAXException se = new SAXException(e);
      se.setStackTrace(e.getStackTrace());
      throw se;
    }
  }

  /**
   * Parses an ownedElement tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startOwnedElement(Attributes attrs) throws ModelElementException
  {
    String id = attrs.getValue(XMI.qName("id"));
    setElement((ModelElement) getElementMap().get(id));
  }

  /**
   * Parses a participant tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startParticipant(Attributes attrs) throws ModelElementException
  {
    if (getRelationship() == null)
      throw new ModelElementException("not within relationship tag while parsing a participant tag");
    RelationElement participant = (RelationElement) getFromRef(attrs);
    if (getRelationship().getSourceParticipant() == null)
    {
      getRelationship().setSourceParticipant(participant);
    }
    else if (getRelationship().getTargetParticipant() == null)
    {
      getRelationship().setTargetParticipant(participant);
    }
    else
    {
      throw new ModelElementException("more than two participant tags found within a relationship tag");
    }
    participant.addRelationship(getRelationship());
  }

  /**
   * Parses a partOf tag.
   * @param attrs element attributes.
   * @throws ModelElementException.
   */
  public void startPartOf(Attributes attrs) throws ModelElementException
  {
    if (getConcept() == null)
      throw new ModelElementException("not within a Concept while parsing a partOf tag");
    getConcept().addPartOf((Role) getFromRef(attrs));
  }

  /**
   * Parses a part tag.
   * @param attrs element attributes.
   * @throws ModelElementException.
   */
  public void startPart(Attributes attrs) throws ModelElementException
  {
    if (getRole() == null)
      throw new ModelElementException("not within a Role while parsing a part tag");
    getRole().addPart((Concept) getFromRef(attrs));
  }

  /**
   * Parses an inheritsFrom tag.
   * @param attrs element attributes.
   * @throws ModelElementException.
   */
  public void startInheritsFrom(Attributes attrs) throws ModelElementException
  {
    if (getRefinedElement() == null)
      throw new ModelElementException("not within a RefinedElement while parsing an inheritsFrom tag");
    ModelElement element = getFromRef(attrs);
    if (!(element instanceof RefinedElement))
      throw new ModelElementException("inheritsFrom is not a refined element", element);
    getRefinedElement().addInheritsFrom((RefinedElement) element);
  }

  /**
   * Parses a superimposesOn tag.
   * @param attrs element attributes.
   * @throws ModelElementException.
   */
  public void startSuperimposesOn(Attributes attrs) throws ModelElementException
  {
    if (getRefinedElement() == null)
      throw new ModelElementException("not within a RefinedElement while parsing a superimposesOn tag");
    ModelElement element = getFromRef(attrs);
    if (!(element instanceof RefinedElement))
      throw new ModelElementException("superimposesOn is not a refined element", element);
    getRefinedElement().addSuperimposesOn((RefinedElement) element);
    ((RefinedElement) element).addSuperimposer(getRefinedElement());
  }

  /**
   * Parses a solutionModel tag.
   * @param attrs element attributes.
   * @throws ModelElementException.
   */
  public void startSolutionModel(Attributes attrs) throws ModelElementException
  {
    if (getSolutionPattern() == null)
      throw new ModelElementException("not within a SolutionPattern while parsing a solutionModel tag");
    ModelElement element = getFromRef(attrs);
    if (!(element instanceof Model))
      throw new ModelElementException("solutionModel is not a model", element);
    getSolutionPattern().setSolutionModel((Model) element);
  }

  /**
   * Parses a targetModel tag.
   * @param attrs element attributes.
   * @throws ModelElementException.
   */
  public void startTargetModel(Attributes attrs) throws ModelElementException
  {
    if (getModelProperty() == null)
      throw new ModelElementException("not within a ModelProperty while parsing a targetModel tag");
    ModelElement element = getFromRef(attrs);
    if (!(element instanceof Model))
      throw new ModelElementException("targetModel is not a model", element);
    getModelProperty().setTargetModel((Model) model);
  }

  /**
   * Characters.
   *
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void characters(char[] ch, int start, int length)
    throws SAXException
  {
  } //characters

  /**
   * Ignorable whitespace.
   *
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void ignorableWhitespace(char[] ch, int start, int length)
    throws SAXException
  {
  } //ignorableWhitespace

  /**
   * End element.
   *
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   */
  public void endElement(String uri, String local, String raw)
    throws SAXException
  {
    if (uri == null)
      throw new SAXException("XML namespace URI may not be null");
    if (uri.equals(XMI.NS_URI))
      endXMIElement(local);
    else
      endCoComposeElement(local);
  } //endElement

  /**
   * End XMI element.
   * @param local local element name.
   * @exception SAXException
   */
  public void endXMIElement(String local) throws SAXException
  {
  }

  /**
   * Start CoCompose element.
   * @param local local element name.
   * @exception SAXException
   */
  public void endCoComposeElement(String local) throws SAXException
  {
    try
    {
      if (local.equals("ownedElement"))
      {
        endOwnedElement();
      }
    }
    catch (ModelElementException e)
    {
      SAXException se = new SAXException(e);
      se.setStackTrace(e.getStackTrace());
      throw se;
    }
  }

  /**
   * Parses an ownedElement tag
   * @throws ModelElementException
   */
  public void endOwnedElement() throws ModelElementException
  {
    if ((getElement() instanceof Namespace) && (getNamespace() != null))
    {
      setElement(getNamespace().getNamespace());
    }
  }

  //
  // DTDHandler methods
  //

  /**
   * DTD declaration.
   *
   * @param name DTD name.
   * @param publicId DTD public ID.
   * @param systemId DTD system ID.
   */
  public void notationDecl(String name, String publicId, String systemId)
    throws SAXException
  {
  } //notationDecl
  
  /**
   * Parses the current ownedElement according to its class and sets
   * the corresponding attributes.
   */
  private void parseOwnedElement()
  {
    if (getElement() == null)
      return;
    if (getElement() instanceof Relationship)
      setRelationship((Relationship) getElement());
    else if (getElement() instanceof Namespace)
    {
      setNamespace((Namespace) getElement());
      if (getElement() instanceof RefinedElement)
      {
        setRefinedElement((RefinedElement) getElement());
        if (getElement() instanceof Role)
          setRole((Role) getElement());
        else if (getElement() instanceof Concept)
          setConcept((Concept) getElement());
      }
      else if (getElement() instanceof SolutionPattern)
        setSolutionPattern((SolutionPattern) getElement());
      else if (getElement() instanceof ModelProperty)
        setModelProperty((ModelProperty) getElement());
    }
  }

  /**
   * Sets the last parsed concept.
   * @param concept the last parsed concept.
   */
  protected void setConcept(Concept concept)
  {
    this.concept = concept;
  }

  /**
   * Gets the last parsed concept.
   * @return the last parsed concept.
   */
  public Concept getConcept()
  {
    return concept;
  }

  /**
   * Sets the last parsed element. 
   * @param element the last parsed element.
   */
  protected void setElement(ModelElement element)
  {
    this.element = element;
    parseOwnedElement();
  }

  /**
   * Gets the last parsed element.
   * @return the last parsed element.
   */
  public ModelElement getElement()
  {
    return element;
  }

  /**
   * Sets the mapping of IDs to elements.
   * @param elementMap the mapping of IDs to elements.
   */
  protected void setElementMap(Map elementMap)
  {
    this.elementMap = elementMap;
  }

  /**
   * Gets the mapping of IDs to elements.
   * @return the mapping of IDs to elements.
   */
  public Map getElementMap()
  {
    return elementMap;
  }

  /**
   * Sets the attribute reader.
   * @param elementReader the attribute reader.
   */
  protected void setElementReader(XMIElementLinkAttributesReader elementReader)
  {
    this.elementReader = elementReader;
  }

  /**
   * Gets the attribute reader.
   * @return the attribute reader.
   */
  public XMIElementLinkAttributesReader getElementReader()
  {
    return elementReader;
  }

  /**
   * Sets the last parsed model.
   * @param model the last parsed model.
   */
  protected void setModel(Model model)
  {
    this.model = model;
  }

  /**
   * Gets the last parsed model.
   * @return the last parsed model.
   */
  public Model getModel()
  {
    return model;
  }

  /**
   * Sets the last parsed namespace.
   * @param namespace the last parsed namespace.
   */
  protected void setNamespace(Namespace namespace)
  {
    this.namespace = namespace;
  }

  /**
   * Gets the last parsed namespace.
   * @return the last parsed namespace.
   */
  public Namespace getNamespace()
  {
    return namespace;
  }

  /**
   * Sets the last parsed relationship.
   * @param relationship the last parsed relationship.
   */
  protected void setRelationship(Relationship relationship)
  {
    this.relationship = relationship;
  }

  /**
   * Gets the last parsed relationship.
   * @return the last parsed relationship.
   */
  public Relationship getRelationship()
  {
    return relationship;
  }

  /**
   * Sets the last parsed role.
   * @param role the last parsed role.
   */
  protected void setRole(Role role)
  {
    this.role = role;
  }

  /**
   * Gets the last parsed role.
   * @return the last parsed role.
   */
  public Role getRole()
  {
    return role;
  }

  /**
   * Sets the last parsed refined element.
   * @param refinedElement the last parsed refined element.
   */
  protected void setRefinedElement(RefinedElement refinedElement)
  {
    this.refinedElement = refinedElement;
  }

  /**
   * Gets the last parsed refined element.
   * @return the last parsed refined element.
   */
  public RefinedElement getRefinedElement()
  {
    return refinedElement;
  }
  
  /**
   * Gets the last parsed solution pattern.
   * @return the last parsed solution pattern.
   */
  public SolutionPattern getSolutionPattern()
  {
    return solutionPattern;
  }

  /**
   * Sets the last parsed solution pattern.
   * @param pattern the last parsed solution pattern.
   */
  protected void setSolutionPattern(SolutionPattern pattern)
  {
    solutionPattern = pattern;
  }

  /**
   * Gets the last parsed model property.
   * @return the last parsed model property.
   */
  public ModelProperty getModelProperty()
  {
    return modelProperty;
  }

  /**
   * Sets the last parsed model property.
   * @param property the last parsed model property.
   */
  protected void setModelProperty(ModelProperty property)
  {
    modelProperty = property;
  }

  /**
   * Gets a ModelElement by IDREF or HREF attribute.
   * @param attrs the XML attributes.
   * @return the ModelElement, or null if not found.
   * @throws ModelElementException if no idref or href attribute is found or if
   * no model element is found.
   */
  protected ModelElement getFromRef(Attributes attrs) throws ModelElementException
  {
    ModelElement element;
    String ref = attrs.getValue(XMI.qName("idref"));
    if (ref == null)
    {
      ref = attrs.getValue(XMI.qName("href"));
      if (ref == null)
        throw new ModelElementException("no idref or href found");
      element = findHref(ref);
    }
    else
    {
      element = (ModelElement) getElementMap().get(ref);
    }
    if (element == null)
      throw new ModelElementException("element with reference " + ref + " not found");
    return element;
  }

  /**
   * Finds the element pointed to by a HREF.
   * @param href the hyperlink to the element.
   * @return the element or null if not found.
   * @throws ModelElementException if href is not a proper hyperlink.
   */
  private ModelElement findHref(String href) throws ModelElementException
  {
    int mark = href.indexOf('#');
    if (mark == -1)
      throw new ModelElementException(href + 
        " is not a proper hyperlink to an element: contains no #");
    if (getElement() == null)
      throw new ModelElementException(
        "No context model element set for the reference " + href);
    Model model = getElement().getModel();
    if (model == null)
      throw new ModelElementException(
        "No context model set for " + getElement().getName() + 
        " when parsing reference " + href);
    String uri = href.substring(0, mark);
    uri = getAbsoluteUri(uri, model.getUri());
    String id = href.substring(mark + 1);
    Model emodel = settings.getSharedModel(uri);
    if (emodel == null)
    {
      try
      {
        ModelReader reader = (ModelReader) Class.forName(
          settings.getSetting("model.reader")).newInstance();
        emodel = reader.read(uri, settings.getFactory());
      }
      catch (Exception e)
      {
        throw new ModelElementException(e);
      }
    }
    settings.addSharedModel(emodel);
    return (ModelElement) emodel.getOwnedIds().get(id);
  }
  
  /**
   * Gets the absolute uri, given the base uri.
   * @param uri the relative (or already absolute) uri.
   * @param baseuri the base uri.
   * @return the absolute uri.
   * @throws ModelElementException if no valid URIs are given.
   */
  private String getAbsoluteUri(String uri, String baseuri)
    throws ModelElementException
  {
    try
    {
      baseuri = baseuri.substring(0, baseuri.lastIndexOf('/') + 1);
      uri = uri.replaceAll(" ", "%20");
      baseuri = baseuri.replaceAll(" ", "%20");
      URI baseUri = new URI(baseuri);
      URI relUri = new URI(uri);
      return baseUri.resolve(relUri).toString();
    }
    catch (URISyntaxException e)
    {
      throw new ModelElementException(e);
    }
  }
}
