/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: XMIModelWriter.java,v 1.4 2004/04/28 12:23:29 dwagelaa Exp $
 * $Log: XMIModelWriter.java,v $
 * Revision 1.4  2004/04/28 12:23:29  dwagelaa
 * + added parsing of ImplementationPattern.content tag
 *
 */

package be.ac.vub.cocompose.io.xmi;

import java.io.IOException;
import java.io.OutputStream;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.ContentHandler;
import org.xml.sax.helpers.AttributesImpl;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.io.xml.SAXWriter;
import be.ac.vub.cocompose.io.FileModelWriter;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelVisitor;

/**
 * Writes the model to an XMI file.
 * @author Dennis Wagelaar
 * @version 0.2.6, 08/03/2004
 * @since 0.2.0
 */
public class XMIModelWriter extends FileModelWriter
{
  private XMIElementWriter elementWriter = new XMIElementWriter();

  /**
   * Constructs an XMIModelWriter.
   */
  public XMIModelWriter()
  {
    SAXWriter saxWriter = new SAXWriter();
    setStreamWriter(saxWriter);
    setXmlWriter(saxWriter);
  }

  /**
   * Writes the model to its URI.
   * @param model - the model to write.
   * @exception IOException - if the model could not be written. 
   * @exception ModelElementException - if a model element could not be written.
   */
  public void write(Model model) throws IOException, ModelElementException 
  {
    super.write(model);
    writeXMI(model);
  }

  /**
   * Writes the model to an OutputStream.
   * @param model - the model to write.
   * @param output - the stream to write to.
   * @exception IOException - if the model could not be written.
   * @exception ModelElementException - if a model element could not be written.
   */
  public void write(Model model, OutputStream output) throws IOException, ModelElementException
  {
    getStreamWriter().setOutputStream(output);
    writeXMI(model);
  }

  /**
   * Gets the xml writer.
   * @return the xml writer.
   */
  public ContentHandler getXmlWriter()
  {
    return elementWriter.getXmlHandler();
  }

  /**
   * Sets the xml writer.
   * @param xmlWriter - the xml writer.
   */
  public void setXmlWriter(ContentHandler xmlWriter)
  {
    elementWriter.setXmlHandler(xmlWriter);
  }

  /**
   * Writes the XMI representation of the given model to the xml writer.
   * @param model - the model to write.
   * @exception IOException - if the model could not be written.
   * @exception ModelElementException - if a model element could not be written.
   */
  public void writeXMI(Model model) throws IOException, ModelElementException 
  {
    ContentHandler xmlWriter = getXmlWriter();
    if (xmlWriter == null)
      throw new IOException("xmlWriter is null");
    try
    {
      xmlWriter.startDocument();
      elementWriter.setRootAttributes(createRootAttributes());
      try
      {
        elementWriter.visit(model);
      }
      catch (java.lang.StackOverflowError e)
      {
        e.printStackTrace();
      }
      xmlWriter.endDocument();
    }
    catch (SAXException e)
    {
      throw new IOException(e.getMessage());
    }
  }

  /**
   * Returns the root attributes for a CoCompose XMI document.
   * @return the root attributes. 
   */
  private Attributes createRootAttributes()
  {
    AttributesImpl rootAtts = new AttributesImpl();
    rootAtts.addAttribute(null, null, XMI.NS + ":version", "CDATA", "2.0");
    rootAtts.addAttribute(null, null, "xmlns:" + XMI.NS, "CDATA", XMI.NS_URI);
    rootAtts.addAttribute(null, null, "xmlns:" + XMI.CoCompose.NS, "CDATA", XMI.CoCompose.NS_URI);
    rootAtts.addAttribute(null, null, "xmlns:" + XMI.Diagram.NS, "CDATA", XMI.Diagram.NS_URI);
    return rootAtts;
  }

  /**
   * Gets the stacked visitor for the contained visitors.
   * @return the stacked visitor.
   */
  public ModelVisitor getStacked()
  {
    return elementWriter.getStacked();
  }

  /**
   * Sets a stacked visitor for the contained visitors.
   * @param stacked - the stacked visitor.
   */
  public void setStacked(ModelVisitor stacked)
  {
    elementWriter.setStacked(stacked);
  }
}
