/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CopyElementLinks.java,v 1.1 2004/04/28 12:24:36 dwagelaa Exp $
 * $Log: CopyElementLinks.java,v $
 * Revision 1.1  2004/04/28 12:24:36  dwagelaa
 * worked on drag-n-drop
 * updated copy/paste
 * updated deep copy operation
 *
 */

package be.ac.vub.cocompose.operations;

import java.util.Hashtable;
import java.util.List;

import org.eclipse.core.internal.runtime.Assert;

import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.VisitedElement;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.Role;

/**
 * Copies the non-containment links of a CoCompose element. Used as a second
 * pass by {@link CopyElement}.
 * @author Dennis Wagelaar
 * @version 0.2.9, 28/04/2004
 * @since 0.2.9
 * @see be.ac.vub.cocompose.operations.CopyElement
 */
public class CopyElementLinks extends DefaultSuperModelVisitor
{
  private VisitedElement copy = null;
  /**
   * Mapping of original elements to their copies: 
   * <pre>Map&lt;VisitedElement |-&gt; VisitedElement&gt;</pre>
   */
  protected Hashtable elementMap = null;
  
  /**
   * Creates a CopyElementLinks.
   * @param elementMap the map of original elements pointing to their copied
   * equivalents. Result from the first copying pass.
   */
  public CopyElementLinks(Hashtable elementMap)
  {
    Assert.isNotNull(elementMap);
    this.elementMap = elementMap;
  }
  
  /**
   * Visits a Namespace.
   * @param namespace - the element to visit.
   * @exception ModelElementException
   */
  public void visitNamespace(Namespace namespace) throws ModelElementException
  {
    super.visitNamespace(namespace);
    Namespace ns = (Namespace) copy;
    List owned = namespace.getOwnedElements();
    for (int i = 0; i < owned.size(); i ++)
    {
      ModelElement element = (ModelElement) owned.get(i);
      setCopy((VisitedElement) elementMap.get(element));
      visit(element);
    }
    copy = ns; // re-set copy to the current element
  }

  /**
   * Visits a Model.
   * @param model - the element to visit.
   * @exception ModelElementException
   */
  public void visitModel(Model model) throws ModelElementException
  {
    super.visitModel(model);
    Model m = (Model) copy;
    m.setDefaultElement((RefinedElement) elementMap.get(model.getDefaultElement()));
  }

  /**
   * Visits a Relationship.
   * @param relationship - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationship(Relationship relationship) throws ModelElementException
  {
    super.visitRelationship(relationship);
    Relationship rel = (Relationship) copy;
    RelationElement source = (RelationElement) elementMap.get(
      relationship.getSourceParticipant());
    RelationElement target = (RelationElement) elementMap.get(
      relationship.getTargetParticipant());
    if (source == null)
    { // copy of source not available
      source = relationship.getSourceParticipant();
    }
    if (target == null)
    { // copy of target not available
      target = relationship.getTargetParticipant();
    }
    rel.setSourceParticipant(source);
    rel.setTargetParticipant(target);
    source.addRelationship(rel);
    target.addRelationship(rel);
  }

  /**
   * Visits a RelationElement.
   * @param relationElement - the element to visit.
   * @exception ModelElementException
   */
//  public void visitRelationElement(RelationElement relationElement) throws ModelElementException
//  {
//    super.visitRelationElement(relationElement);
//    RelationElement relel = (RelationElement) copy;
//    List rels = relationElement.getRelationships();
//    for (int i = 0; i < rels.size(); i++)
//    {
//      Relationship rel = (Relationship) elementMap.get(rels.get(i));
//      if (rel != null)
//      { // copy of relationship available
//        relel.addRelationship(rel);
//      }
//    }
//  }

  /**
   * Visits a RefinedElement.
   * @param refinedElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
  {
    super.visitRefinedElement(refinedElement);
    RefinedElement refel = (RefinedElement) copy;
    if (refinedElement.getDefaultElementOf() != null)
    {
      Model model = (Model) elementMap.get(refinedElement.getDefaultElementOf());
      if (model == null)
      { // copy of model not available
        model = refinedElement.getDefaultElementOf();
      }
      refel.setDefaultElementOf(model);
    }
  }

  /**
   * Visits a concept.
   * @param concept - the element to visit.
   * @exception ModelElementException
   */
  public void visitConcept(Concept concept) throws ModelElementException
  {
    super.visitConcept(concept);
    Concept c = (Concept) copy;
    List partOf = concept.getPartOf();
    for (int i = 0; i < partOf.size(); i++)
    {
      Role role = (Role) elementMap.get(partOf.get(i));
      if (role == null)
      { // copy of role not available
        role = (Role) partOf.get(i);
      }
      c.addPartOf(role);
    }
  }

  /**
   * Visits a Role.
   * @param role - the element to visit.
   * @exception ModelElementException
   */
  public void visitRole(Role role) throws ModelElementException
  {
    super.visitRole(role);
    Role r = (Role) copy;
    List parts = role.getParts();
    for (int i = 0; i < parts.size(); i++)
    {
      Concept concept = (Concept) elementMap.get(parts.get(i));
      if (concept == null)
      { // copy of concept not available
        concept = (Concept) parts.get(i);
      }
      r.addPart(concept);
    }
  }

  /**
   * Sets the generated copy.
   * @param copy the generated copy.
   */
  public void setCopy(VisitedElement copy)
  {
    Assert.isNotNull(copy);
    this.copy = copy;
  }
}
